Module iterator_wrapper

Branded wrapper to mark iterators that yield a normalized sequence of ranges, ranges that are known to contain sorted and disjoint non-empty ranges.

Structs

NormalizedRangeIterWrapper
This crate uses NormalizedRangeIterWrapper internally to brand regular Iterators that are known to yield normalized ranges. The wrapper itself is an Iterator, and preserves DoubleEndedIterator and ExactSizeIterator if the underlying iterator implements them.