Branded wrapper to mark iterators that yield a normalized sequence of ranges, ranges that are known to contain sorted and disjoint non-empty ranges.
Structs
- Normalized
Range Iter Wrapper - This
crateusesNormalizedRangeIterWrapperinternally to brand regularIterators that are known to yield normalized ranges. The wrapper itself is anIterator, and preservesDoubleEndedIteratorandExactSizeIteratorif the underlying iterator implements them.